@@ -19,11 +19,6 @@
 # Boston, MA 02111-1307, USA.
 #
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import copy
 import operator
 from itertools import chain
@@ -31,6 +26,7 @@ from collections import OrderedDict
 
 from . import message
 
+from .sourcescanner import CTYPE_TYPEDEF, CSYMBOL_TYPE_TYPEDEF
 from .message import Position
 from .utils import to_underscores
 
@@ -202,6 +198,7 @@ class TypeUnknown(Type):
     def __init__(self):
         Type.__init__(self, _target_unknown=True)
 
+
 # Fundamental types, two special ones
 TYPE_NONE = Type(target_fundamental='none', ctype='void')
 TYPE_ANY = Type(target_fundamental='gpointer', ctype='gpointer')
@@ -321,6 +318,8 @@ type_names['gssize'] = TYPE_SSIZE
 type_names['gintptr'] = TYPE_INTPTR
 type_names['guintptr'] = TYPE_UINTPTR
 type_names['gconstpointer'] = TYPE_ANY
+type_names['grefcount'] = TYPE_INT
+type_names['gatomicrefcount'] = TYPE_INT
 
 # We used to support these; continue to do so
 type_names['any'] = TYPE_ANY
@@ -562,6 +561,7 @@ properties."""
         self.deprecated = None
         self.deprecated_doc = None
         self.doc = None
+        self.doc_position = None
 
 
 class Node(Annotated):
@@ -629,9 +629,23 @@ GIName.  It's possible for nodes to contain or point to other nodes."""
     def add_file_position(self, position):
         self.file_positions.add(position)
 
+    def get_main_position(self):
+        if not self.file_positions:
+            return None
+
+        res = None
+        for position in self.file_positions:
+            if position.is_typedef:
+                res = position
+            else:
+                return position
+
+        return res
+
     def add_symbol_reference(self, symbol):
         if symbol.source_filename:
-            self.add_file_position(Position(symbol.source_filename, symbol.line))
+            self.add_file_position(Position(symbol.source_filename, symbol.line,
+                is_typedef=symbol.type in (CTYPE_TYPEDEF, CSYMBOL_TYPE_TYPEDEF)))
 
     def walk(self, callback, chain):
         res = callback(self, chain)
@@ -714,6 +728,14 @@ class Callable(Node):
         raise ValueError("Unknown argument %s" % (name, ))
 
 
+class FunctionMacro(Node):
+    def __init__(self, name, parameters, symbol):
+        Node.__init__(self, name)
+        self.symbol = symbol
+        self.parameters = parameters
+        self.introspectable = False
+
+
 class Function(Callable):
 
     def __init__(self, name, retval, parameters, throws, symbol):
@@ -854,7 +876,7 @@ class TypeContainer(Annotated):
         self.direction = direction
         if transfer is not None:
             self.transfer = transfer
-        elif typenode.is_const:
+        elif typenode and typenode.is_const:
             self.transfer = PARAM_TRANSFER_NONE
         else:
             self.transfer = None
