@@ -867,6 +867,13 @@ void       regress_forced_method (RegressTestObj *obj);
 _GI_TEST_EXTERN
 void regress_test_array_fixed_out_objects (RegressTestObj ***objs);
 
+#if (defined(__GNUC__) || defined(__clang__)) && defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
+_GI_TEST_EXTERN
+void regress_test_array_static_in_int (int x[static 10]);
+#else
+_GI_TEST_EXTERN
+void regress_test_array_static_in_int (int x[10]);
+#endif
 
 _GI_TEST_EXTERN
 void regress_test_obj_torture_signature_0 (RegressTestObj    *obj,
